package com.atguigu2.statement.crud;

import com.atguigu3.util.JDBCUtils;

import java.lang.reflect.Field;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.util.Scanner;

/**
 * @Author: ZZP
 * @Description:
 * 演示 PreparedStatement 替换 Statement ，解决 SQL 注入问题
 *
 * 除了解决 Statement 的拼串、sql问题之外，PreparedStatement 还有哪些好处呢？
 *      1.PreparedStatement 操作 Blob 的数据，而 Statement 做不到
 *      2.PreparedStatement 可以实现更高效的批量操作
 *
 * @Date: Created in 16:18 2021/6/17
 * @Version: 1.0
 **/
public class PreparedStatementTest {
    public static void main(String[] args) {
        testLogin();
    }

    public static void testLogin() {
        // 使用Statement的弊端：需要拼写sql语句，并且存在SQL注入的问题
        Scanner scan = new Scanner(System.in);

        System.out.print("用户名：");
        String userName = scan.nextLine();
        System.out.print("密   码：");
        String password = scan.nextLine();

        //SELECT user,password FROM user_table WHERE USER = '1' or ' AND PASSWORD = '=1 or '1' = '1';
        String sql = "SELECT user,password FROM user_table WHERE user = ? and password = ?";
        User user = getInstance(User.class, sql, userName, password);
        if (user != null) {
            System.out.println("登陆成功!");
        } else {
            System.out.println("用户名或密码错误！");
        }
    }

    /**
     * @Author: ZZP
     * @Date:2021/6/17
     * @Description: 针对于不同表的通用的查询操作，返回表中的一条记录
     */
    public static <T> T getInstance(Class<T> clazz, String sql, Object... args) {
        Connection conn = null;
        PreparedStatement ps = null;
        ResultSet resultSet = null;
        try {
            conn = JDBCUtils.getConnection();

            // 2.预编译 sql 语句，返回 PreparedStatement 的实例
            ps = conn.prepareStatement(sql);

            // 3.填充占位符
            for (int i = 0; i < args.length; i++) {
                ps.setObject(i + 1, args[i]);
            }

            // 4.执行并返回结果集
            resultSet = ps.executeQuery();

            // 获取结果集的元数据：ResultSetMetaData
            ResultSetMetaData rsmd = resultSet.getMetaData();

            // 通过 ResultSetMetaData 获取结果集中的列数
            int columnCount = rsmd.getColumnCount();

            // 5.处理结果集
            if (resultSet.next()) {
                T t = clazz.newInstance();

                // 处理结果集一行数据中的每一个列
                for (int i = 0; i < columnCount; i++) {
                    // 获取列值
                    Object columnValue = resultSet.getObject(i + 1);

                    // 获取每个列的列名 getColumnName()
                    // 获取每个列的别名 getColumnLabel()
                    String columnLabel = rsmd.getColumnLabel(i + 1);

                    // 给 columnName 对象指定的某个属性，赋值为 columnValue : 通过反射
                    Field field = clazz.getDeclaredField(columnLabel);
                    field.setAccessible(true);
                    field.set(t, columnValue);
                }
                return t;
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            // 关闭资源
            JDBCUtils.closeResource(conn, ps, resultSet);
        }
        return null;
    }
}
