package com.atguigu5.blob;

import com.atguigu3.util.JDBCUtils;

import java.sql.Connection;
import java.sql.PreparedStatement;

/**
 * @Author: ZZP
 * @Description:
 * 使用 PreparedStatement 实现批量数据的操作
 *
 * update / delete 本身就具有批量操作的效果
 * 此时的批量操作，主要指的是批量插入。使用 PreparedStatement 如何实现更高效的批量插入
 *
 * 题目：向 goods 表中插入 20000 条数据
 * 方式一：使用 Statement
 *
 *
 * @Date: Created in 15:20 2021/6/18
 * @Version: 1.0
 **/
public class InsertTest {
    public static void main(String[] args) {
        //testInsert1();

        //testInsert2();

        testInsert3();
    }

    /**
     * @Author: ZZP
     * @Date:2021/6/18
     * @Description: 批量插入的方式二：使用 PreparedStatement
     */
    public static void testInsert1() {
        Connection conn = null;
        PreparedStatement ps = null;
        try {
            long start = System.currentTimeMillis();

            conn = JDBCUtils.getConnection();
            String sql = "insert into goods(name)value(?)";
            ps = conn.prepareStatement(sql);
            for (int i = 1; i <= 20000; i++) {
                ps.setObject(1,"name_" + i);

                ps.execute();
            }
            long end = System.currentTimeMillis();

            System.out.println("花费的时间为：" + (end - start));
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            JDBCUtils.closeResource(conn,ps);
        }
    }

    /**
     * @Author: ZZP
     * @Date:2021/6/18
     * @Description:
     * 批量插入的方式三：
     *      1.addBatch()、executeBatch()、clearBatch()
     *      2.mysql服务器默认是关闭批处理的，我们需要通过一个参数，让mysql开启批处理的支持。 ?rewriteBatchedStatements=true 写在配置文件的url后面
     *      3.使用更新的mysql 驱动：mysql-connector-java-5.1.37-bin.jar
     */
    public static void testInsert2() {
        Connection conn = null;
        PreparedStatement ps = null;
        try {
            long start = System.currentTimeMillis();

            conn = JDBCUtils.getConnection();
            String sql = "insert into goods(name)value(?)";
            ps = conn.prepareStatement(sql);
            for (int i = 1; i <= 20000; i++) {
                ps.setObject(1,"name_" + i);

                // 1."攒" sql
                ps.addBatch();

                if (i % 500 == 0) {
                    // 2.执行
                    ps.executeBatch();

                    // 3.清空batch
                    ps.clearBatch();
                }
            }
            long end = System.currentTimeMillis();

            System.out.println("花费的时间为：" + (end - start));
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            JDBCUtils.closeResource(conn,ps);
        }
    }

    /**
     * @Author: ZZP
     * @Date:2021/6/18
     * @Description: 批量插入的方式四：设置不允许自动提交数据
     */
    public static void testInsert3() {
        Connection conn = null;
        PreparedStatement ps = null;
        try {
            long start = System.currentTimeMillis();

            conn = JDBCUtils.getConnection();

            // 设置不允许自动提交数据
            conn.setAutoCommit(false);

            String sql = "insert into goods(name)value(?)";
            ps = conn.prepareStatement(sql);
            for (int i = 1; i <= 20000; i++) {
                ps.setObject(1,"name_" + i);

                // 1."攒" sql
                ps.addBatch();

                if (i % 500 == 0) {
                    // 2.执行 batch
                    ps.executeBatch();

                    // 3.清空 batch
                    ps.clearBatch();
                }
            }
            // 提交数据
            conn.commit();

            long end = System.currentTimeMillis();

            System.out.println("花费的时间为：" + (end - start));
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            JDBCUtils.closeResource(conn,ps);
        }
    }
}
