package com.atguigu1.transaction;

import com.atguigu1.util.JDBCUtils;
import org.junit.Test;

import java.lang.reflect.Field;
import java.sql.*;

/**
 * @Author: ZZP
 * @Date: Created in 20:33 2021/6/18
 * @Version: 1.0
 * @Description:
 * 1.什么叫数据库事务？
 *      事物：一组逻辑操作单元，使数据从一种状态变到另外一种状态
 *          > 一组逻辑单元：一个或多个 DML 操作
 *
 * 2.事务处理（事务操作）原则：
 *      保证所有事务都作为一个工作单元来执行，即使出现了故障，都不能改变这种执行方式。
 *      当在一个事务中执行多个操作时，要么所有的事务都被提交(commit)，那么这些修改就永久地保存下来；
 *      要么数据库管理系统将放弃所作的所有修改，整个事务回滚(rollback)到最初状态。
 *
 * 3.数据一旦提交，就不可回滚
 *
 * 4.哪些操作会导致数据的自动提交？
 *      > DDL 操作一旦执行，都会提交
 *      > DML 默认情况下，一旦执行，就会自动提交
 *          > 我们可以通过 set autocommit = false; 的方式取消 DML 操作的自动提交
 *      > 默认在关闭连接时，会自动的提交数据
 **/
public class TransactionTest {
    public static void main(String[] args) {
        //testUpdate();

        //testUpdateWithTx();
    }

    /**
     * @Author: ZZP
     * @Date:2021/6/18
     * @Version: 1.0
     * @Description:
     * 针对于数据表 user_table来说：
     *      AA 用户给 BB 用户转账 100
     *
     * update user_table set balance = balance - 100 where user = 'AA'
     * update user_table set balance = balance + 100 where user = 'BB'
     */
    public static void testUpdate() {
        String sql1 = "update user_table set balance = balance - 100 where user = ?";
        update(sql1, "AA");

        // 模拟网络异常
        System.out.println(10 / 0);

        String sql2 = "update user_table set balance = balance + 100 where user = ?";
        update(sql2, "BB");

        System.out.println("转账成功！");
    }

    /**
     * @Author: ZZP
     * @Date:2021/6/18
     * @Version: 1.0 - 未考虑数据库事物情况下的转账操作
     * @Description: 通用的增删改操作
     */
    public static int update(String sql, Object... args) { // sql中的占位符的个数与可变形参的长度相同
        Connection conn = null;
        PreparedStatement ps = null;
        try {
            // 1.获取数据库的连接
            conn = JDBCUtils.getConnection();

            // 2.预编译 sql 语句，返回 PreparedStatement 的实例
            ps = conn.prepareStatement(sql);

            // 3.填充占位符
            for (int i = 0; i < args.length; i++) {
                ps.setObject(i + 1, args[i]);
            }

            // 4.执行操作
            return ps.executeUpdate();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            // 5.关闭资源
            JDBCUtils.closeResource(conn, ps);
        }
        return 0;
    }

    /**
     * @Author: ZZP
     * @Date:2021/6/18
     * @Version: 2.0
     * @Description:
     * 针对于数据表 user_table来说：
     *      AA 用户给 BB 用户转账 100
     *
     * update user_table set balance = balance - 100 where user = 'AA'
     * update user_table set balance = balance + 100 where user = 'BB'
     */
    public static void testUpdateWithTx() {
        Connection conn = null;
        try {
            conn = JDBCUtils.getConnection();

            // 取消数据的自动提交功能
            conn.setAutoCommit(false);

            String sql1 = "update user_table set balance = balance - 100 where user = ?";
            update(conn, sql1, "AA");

            // 模拟网络异常
            System.out.println(10 / 0);

            String sql2 = "update user_table set balance = balance + 100 where user = ?";
            update(conn, sql2, "BB");

            System.out.println("转账成功！");

            // 提交数据
            conn.commit();
        } catch (Exception e) {
            e.printStackTrace();
            // 回滚数据
            try {
                conn.rollback();
                System.out.println("转账失败！");
            } catch (SQLException throwables) {
                throwables.printStackTrace();
            }
        } finally {
            try {
                // 修改其为自动提交数据
                // 主要针对于使用数据库连接池的使用
                conn.setAutoCommit(true);
            } catch (SQLException throwables) {
                throwables.printStackTrace();
            }

            JDBCUtils.closeResource(conn, null);
        }
    }

    /**
     * @Author: ZZP
     * @Date:2021/6/18
     * @Version: 2.0 - 考虑数据库事物情况下的转账操作
     * @Description:
     */
    public static int update(Connection conn, String sql, Object... args) { // sql中的占位符的个数与可变形参的长度相同
        PreparedStatement ps = null;
        try {
            // 1.预编译 sql 语句，返回 PreparedStatement 的实例
            ps = conn.prepareStatement(sql);

            // 2.填充占位符
            for (int i = 0; i < args.length; i++) {
                ps.setObject(i + 1, args[i]);
            }

            // 3.执行操作
            return ps.executeUpdate();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            // 4.关闭资源
            JDBCUtils.closeResource(null, ps);
        }
        return 0;
    }

    //******************************************************************************************************************

    @Test
    public void test() {
        Connection connection = null;
        try {
            connection = JDBCUtils.getConnection();
            System.out.println(connection.getTransactionIsolation());
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (connection != null) {
                try {
                    connection.close();
                } catch (SQLException throwables) {
                    throwables.printStackTrace();
                }
            }
        }
    }

    @Test
    public void testTransactionSelect() throws Exception {
        Connection conn = JDBCUtils.getConnection();

        // 设置当前的隔离级别
        conn.setTransactionIsolation(Connection.TRANSACTION_READ_UNCOMMITTED);
        // 获取当前的隔离级别
        System.out.println(conn.getTransactionIsolation());
        // 取消自动提交数据
        conn.setAutoCommit(false);

        String sql = "select user, password, balance from user_table where user = ?";
        User user = getInstance(conn, User.class, sql, "CC");

        System.out.println(user);
    }

    @Test
    public void testTransactionUpdate() throws Exception {
        Connection conn = JDBCUtils.getConnection();

        // 取消自动提交数据
        conn.setAutoCommit(false);

        String sql = "update user_table set balance = ? where user = ?";
        update(conn, sql, 5000, "CC");

        Thread.sleep(15000);

        System.out.println("修改结束");
    }

    /**
     * @Author: ZZP
     * @Date:2021/6/19
     * @Version: 2.0 - 考虑上事务
     * @Description: 针对于不同表的通用的查询操作，返回表中的一条记录
     */
    public static <T> T getInstance(Connection conn, Class<T> clazz, String sql, Object... args) {
        PreparedStatement ps = null;
        ResultSet resultSet = null;
        try {
            // 1.预编译 sql 语句，返回 PreparedStatement 的实例
            ps = conn.prepareStatement(sql);

            // 2.填充占位符
            for (int i = 0; i < args.length; i++) {
                ps.setObject(i + 1, args[i]);
            }

            // 3.执行并返回结果集
            resultSet = ps.executeQuery();

            // 获取结果集的元数据：ResultSetMetaData
            ResultSetMetaData rsmd = resultSet.getMetaData();

            // 通过 ResultSetMetaData 获取结果集中的列数
            int columnCount = rsmd.getColumnCount();

            // 4.处理结果集
            if (resultSet.next()) {
                T t = clazz.newInstance();

                // 处理结果集一行数据中的每一个列
                for (int i = 0; i < columnCount; i++) {
                    // 获取列值
                    Object columnValue = resultSet.getObject(i + 1);

                    // 获取每个列的列名 getColumnName()
                    // 获取每个列的别名 getColumnLabel()
                    String columnLabel = rsmd.getColumnLabel(i + 1);

                    // 给 columnName 对象指定的某个属性，赋值为 columnValue : 通过反射
                    Field field = clazz.getDeclaredField(columnLabel);
                    field.setAccessible(true);
                    field.set(t, columnValue);
                }
                return t;
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            // 关闭资源
            JDBCUtils.closeResource(null, ps, resultSet);
        }
        return null;
    }
}
