package com.atguigu5.dbutils;

import com.atguigu2.bean.Customer;
import com.atguigu4.util.JDBCUtils;
import org.apache.commons.dbutils.QueryRunner;
import org.apache.commons.dbutils.ResultSetHandler;
import org.apache.commons.dbutils.handlers.*;
import org.junit.Test;

import java.sql.Connection;
import java.sql.Date;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;
import java.util.Map;

/**
 * @Author: ZZP
 * @Date: Created in 19:17 2021/6/19
 * @Version: 1.0
 * @Description: commons-dbutils 是 Apache 组织提供的一个开源 JDBC工具类库，封装了针对于数据库的增删改查操作
 **/
public class QueryRunnerTest {
    @Test
    public void testInsert() throws SQLException {
        Connection conn = null;
        try {
            QueryRunner runner = new QueryRunner();

            conn = JDBCUtils.getConnection3();

            String sql = "insert into customers(name ,email,birth) value (?,?,?)";

            int insertCount = runner.update(conn, sql, "蔡徐坤", "caixukun@qq.com", "1997-01-01");

            System.out.println("添加了 " + insertCount + " 条记录");
        } catch (SQLException throwables) {
            throwables.printStackTrace();
        } finally {
            JDBCUtils.closeResource(conn, null);
        }
    }

    /**
     * @Author: ZZP
     * @Date:2021/6/19
     * @Version: 1.0
     * @Description: BeanHandler:是 ResultSetHandler 接口的实现类，用于封装表中的一条记录
     */
    @Test
    public void testQuery1() {
        Connection conn = null;
        try {
            QueryRunner runner = new QueryRunner();

            conn = JDBCUtils.getConnection3();

            String sql = "select id, name, email, birth from customers where id = ?";

            BeanHandler<Customer> handler = new BeanHandler<>(Customer.class);

            Customer customer = runner.query(conn, sql, handler, 23);

            System.out.println(customer);
        } catch (SQLException throwables) {
            throwables.printStackTrace();
        } finally {
            JDBCUtils.closeResource(conn, null);
        }
    }

    /**
     * @Author: ZZP
     * @Date:2021/6/19
     * @Version: 1.0
     * @Description: BeanListHandler:是 ResultSetHandler 接口的实现类，用于封装表中的多条记录构成的集合
     */
    @Test
    public void testQuery2() {
        Connection conn = null;
        try {
            QueryRunner runner = new QueryRunner();

            conn = JDBCUtils.getConnection3();

            String sql = "select id, name, email, birth from customers where id < ?";

            BeanListHandler<Customer> listHandler = new BeanListHandler<Customer>(Customer.class);

            List<Customer> customerList = runner.query(conn, sql, listHandler, 23);

            customerList.forEach(System.out::println);
        } catch (SQLException throwables) {
            throwables.printStackTrace();
        } finally {
            JDBCUtils.closeResource(conn, null);
        }
    }

    /**
     * @Author: ZZP
     * @Date:2021/6/19
     * @Version: 1.0
     * @Description: MapHandler:是 ResultSetHandler 接口的实现类，对应表中的一条记录。将字段及相应字段的值作为 map 中的 key 和 value
     */
    @Test
    public void testQuery3() {
        Connection conn = null;
        try {
            QueryRunner runner = new QueryRunner();

            conn = JDBCUtils.getConnection3();

            String sql = "select id, name, email, birth from customers where id = ?";

            MapHandler mapHandler = new MapHandler();

            Map<String, Object> customerMap = runner.query(conn, sql, mapHandler, 23);

            System.out.println(customerMap);
        } catch (SQLException throwables) {
            throwables.printStackTrace();
        } finally {
            JDBCUtils.closeResource(conn, null);
        }
    }

    /**
     * @Author: ZZP
     * @Date:2021/6/19
     * @Version: 1.0
     * @Description: MapHandler:是 ResultSetHandler 接口的实现类，对应表中的多条记录。将字段及相应字段的值作为 map 中的 key 和 value。将这些 map 添加到 List 中
     */
    @Test
    public void testQuery4() {
        Connection conn = null;
        try {
            QueryRunner runner = new QueryRunner();

            conn = JDBCUtils.getConnection3();

            String sql = "select id, name, email, birth from customers where id < ?";

            MapListHandler mapListHandler = new MapListHandler();

            List<Map<String, Object>> customerMapList = runner.query(conn, sql, mapListHandler, 23);

            customerMapList.forEach(System.out::println);
        } catch (SQLException throwables) {
            throwables.printStackTrace();
        } finally {
            JDBCUtils.closeResource(conn, null);
        }
    }

    /**
     * @Author: ZZP
     * @Date:2021/6/19
     * @Version: 1.0
     * @Description: ScalarHandler:用于查询特殊值
     */
    @Test
    public void testQuery5() {
        Connection conn = null;
        try {
            QueryRunner runner = new QueryRunner();

            conn = JDBCUtils.getConnection3();

            String sql = "select count(*) from customers";

            ScalarHandler scalarHandler = new ScalarHandler();

            Object customerCount = runner.query(conn, sql, scalarHandler);

            System.out.println(customerCount);
        } catch (SQLException throwables) {
            throwables.printStackTrace();
        } finally {
            JDBCUtils.closeResource(conn, null);
        }
    }

    /**
     * @Author: ZZP
     * @Date:2021/6/19
     * @Version: 1.0
     * @Description: 自定义 ResultSetHandler 的实现类
     */
    @Test
    public void testQuery7() {
        Connection conn = null;
        try {
            QueryRunner runner = new QueryRunner();

            conn = JDBCUtils.getConnection3();

            String sql = "select max(birth) from customers";

            ScalarHandler scalarHandler = new ScalarHandler();

            Object maxBirth = runner.query(conn, sql, scalarHandler);

            System.out.println(maxBirth);
        } catch (SQLException throwables) {
            throwables.printStackTrace();
        } finally {
            JDBCUtils.closeResource(conn, null);
        }
    }

    /**
     * @Author: ZZP
     * @Date:2021/6/19
     * @Version: 1.0
     * @Description: ScalarHandler:用于查询特殊值
     */
    @Test
    public void testQuery6() {
        Connection conn = null;
        try {
            QueryRunner runner = new QueryRunner();

            conn = JDBCUtils.getConnection3();

            String sql = "select id, name, email, birth from customers where id = ?";

            ResultSetHandler handler = new ResultSetHandler<Customer>() {

                @Override
                public Customer handle(ResultSet resultSet) throws SQLException {
                    //return new Customer(12,"成龙","jacky@qq.com",new Date(54635313153153L));

                    if (resultSet.next()) {
                        int id = resultSet.getInt("id");
                        String name = resultSet.getString("name");
                        String email = resultSet.getString("email");
                        Date birth = resultSet.getDate("birth");
                        Customer customer = new Customer(id, name, email, birth);

                        return customer;
                    }
                    return null;
                }
            };

            Object customer = runner.query(conn, sql, handler, 23);

            System.out.println(customer);
        } catch (SQLException throwables) {
            throwables.printStackTrace();
        } finally {
            JDBCUtils.closeResource(conn, null);
        }
    }
}
